/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
#ifdef HAVE_MNTENT_H
#include <mntent.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <libxfce4util/libxfce4util.h>

#include "ltop_os_info.h"

#define MNT "/proc/mounts"

/*
 * Public API
 */
LtopOSInfo *
ltop_os_info_get ()
{
    FILE *fp;
    struct mntent *fs;
    gboolean have_home = FALSE;
    gboolean have_cow = FALSE;
    gboolean have_rofsbis = FALSE;

    XfceRc *rc;
    gchar *version = NULL;
    gchar **tokens;


    LtopOSInfo *info = g_new(LtopOSInfo, 1);
    info->root_device = NULL;
    info->persistence_mount = NULL;

    /* get root device and persistence */
    fp = setmntent (MNT, "r");
    if (G_LIKELY (fp)) {
        while ((fs = getmntent (fp)) != NULL)
        {
            if (strcmp (fs->mnt_dir, "/cdrom") == 0) {
                /* /dev/[hs]d? */
                info->root_device = g_strndup (fs->mnt_fsname, 8);
            }
            if (strncmp (fs->mnt_dir, "/home", 5) == 0) {
                info->persistence_mount = g_strdup (fs->mnt_dir);
                have_home = TRUE;
            }
            if (strcmp (fs->mnt_dir, "/cdrom/.cow") == 0) {
                info->persistence_mount = g_strdup ("/");
                have_cow = TRUE;
            }
            if (strncmp (fs->mnt_dir, "/rofsbis", 8) == 0) {
                info->persistence_mount = g_strdup ("/");
                have_rofsbis = TRUE;
            }
        }
        endmntent (fp);

        info->persistence = LTOP_PERSISTENCE_ERROR;
        if (have_home) {
            info->persistence = LTOP_PERSISTENCE_HOME;
        }
        else if (have_cow) {
            if (have_rofsbis)
                info->persistence = LTOP_PERSISTENCE_NONE;
            else
                info->persistence = LTOP_PERSISTENCE_FULL;
        }
    }

    /* OS version and machine type */
    rc = xfce_rc_simple_open ("/etc/lsb-release", TRUE);
    if (G_LIKELY (rc)) {
        version = g_strdup (xfce_rc_read_entry (rc, "LINUTOP_CODENAME", ""));
        xfce_rc_close (rc);

        /* /!\ 0 and 4 are empty strings if the regex matches */
        tokens = g_regex_split_simple ("(.*)(l2|l3|eee)", version, 0, 0);
        if (g_strv_length (tokens) == 4) {
            info->version = g_strdup (tokens[1]);
            info->full_version = g_strdup (version);
            if (! strcmp ("l2", tokens[2]))
                info->machine = LTOP_MACHINE_L2;
            else if (! strcmp ("l3", tokens[2]))
                info->machine = LTOP_MACHINE_L3;
            else if (! strcmp ("eee", tokens[2]))
                info->machine = LTOP_MACHINE_EEE;
            else {
                info->machine = LTOP_MACHINE_ERROR;
            }
        }
        g_strfreev (tokens);
        g_free (version);
    }

    /* laptop */
    info->laptop = system("laptop-detect >/dev/nulli 2>&1") ? FALSE : TRUE;

    return info;
}

void
ltop_os_info_free (LtopOSInfo *info)
{
    g_free (info->version);
    info->version = NULL;
    g_free (info->full_version);
    info->full_version = NULL;
    g_free (info->root_device);
    info->root_device = NULL;
    g_free (info->persistence_mount);
    info->persistence_mount = NULL;

    g_free (info);
}
